@@ -6,6 +6,7 @@ local class = require 'pl.class'
 local utils = require 'pl.utils'
 local List = require 'pl.List'
 local Map = require 'pl.Map'
+local text = require 'pl.text'
 
 local doc = {}
 local global = require 'ldoc.builtin.globals'
@@ -23,9 +24,9 @@ local TAG_MULTI,TAG_ID,TAG_SINGLE,TAG_TYPE,TAG_FLAG,TAG_MULTI_LINE = 'M','id','S
 --  - 'T' tags which represent a type, like 'function' (TAG_TYPE)
 local known_tags = {
    param = 'M', see = 'M', comment = 'M', usage = 'ML', ['return'] = 'M', field = 'M', author='M',set='M';
-   class = 'id', name = 'id', pragma = 'id', alias = 'id', within = 'id',
+   class = 'id', name = 'id', pragma = 'id', alias = 'id',
    copyright = 'S', summary = 'S', description = 'S', release = 'S', license = 'S',
-   fixme = 'S', todo = 'S', warning = 'S', raise = 'S', charset = 'S',
+   fixme = 'S', todo = 'S', warning = 'S', raise = 'S', charset = 'S', within = 'S',
    ['local'] = 'N', export = 'N', private = 'N', constructor = 'N', static = 'N',include = 'S',
    -- project-level
    module = 'T', script = 'T', example = 'T', topic = 'T', submodule='T', classmod='T', file='T',
@@ -141,7 +142,7 @@ function doc.expand_annotation_item (tags, last_item)
          tags:add('name',item_name..'-'..tag..acount)
          acount = acount + 1
          return true
-      elseif tag == 'return' then
+      elseif tags.error and tag == 'return' then
          last_item:set_tag(tag,value)
       end
    end
@@ -294,7 +295,7 @@ function File:finish()
             end
             item.display_name = display_name
             this_mod.section = item
-            this_mod.kinds:add_kind(display_name,display_name..' ',nil,item)
+            this_mod.kinds:add_kind(display_name,display_name,nil,item)
             this_mod.sections:append(item)
             this_mod.sections.by_name[lookup_name:gsub('%A','_')] = item
          end
@@ -536,13 +537,17 @@ function Item.check_tag(tags,tag, value, modifiers)
          if avalue then value = avalue..' '..value end
          if amod then
             modifiers = modifiers or {}
+            local value_tokens = utils.split(value)
             for m,v in pairs(amod) do
-               local idx = v:match('^%$(%d+)')
+               local idx = tonumber(v:match('^%$(%d+)'))
                if idx then
                   v, value = value:match('(%S+)(.*)')
+               --   v = value_tokens[idx]
+               --   value_tokens[idx] = ''
                end
                modifiers[m] = v
             end
+            -- value = table.concat(value_tokens, ' ')
          end
       else -- has to be a function that at least returns tag, value
          return alias(tags,value,modifiers)
@@ -616,6 +621,12 @@ function Item:finish()
    if tags.see then
       tags.see = tools.identifier_list(tags.see)
    end
+   if self.usage then
+      for i = 1,#self.usage do
+         local usage = self.usage[i]:gsub('^%s*\n','')
+         self.usage[i] = text.dedent(usage)
+      end
+   end
    if  doc.project_level(self.type) then
       -- we are a module, so become one!
       self.items = List()
@@ -913,6 +924,7 @@ end
 local struct_return_type = '*'
 
 function Item:build_return_groups()
+   local quote = tools.quote
    local modifiers = self.modifiers
    local retmod = modifiers['return']
    local groups = List()
@@ -1019,6 +1031,7 @@ function Item:warning(msg)
    if type(file) == 'table' then require 'pl.pretty'.dump(file); file = '?' end
    file = file or '?'
    io.stderr:write(file,':',self.lineno or '1',': ',self.name or '?',': ',msg,'\n')
+   Item.had_warning = true
    return nil
 end
 
@@ -1197,7 +1210,8 @@ end
 -- and try to to resolve this.
 function Module:resolve_references(modules)
    local found = List()
-   for item in self.items:iter() do
+   -- Resolve see references in item. Can be Module or Item type.
+   local function resolve_item_references(item)
       local see = item.tags.see
       if see then -- this guy has @see references
          item.see = List()
@@ -1212,6 +1226,11 @@ function Module:resolve_references(modules)
          end
       end
    end
+
+   resolve_item_references(self); -- Resolve module-level see references.
+   for item in self.items:iter() do
+      resolve_item_references(item); -- Resolve item-level see references.
+   end
    -- mark as found, so we don't waste time re-searching
    for f in found:iter() do
       f[1].tags.see:remove_value(f[2])
@@ -1244,10 +1263,17 @@ local function dump_tags (tags)
    end
 end
 
+-- ANSI colour codes for making important stuff BOLD
+-- (but not on Windows)
+local bold,default = '\x1B[1m','\x1B[0m'
+if utils.dir_separator == '\\' then
+   bold,default = '',''
+end
+
 function Module:dump(verbose)
    if not doc.project_level(self.type) then return end
    print '----'
-   print(self.type..':',self.name,self.summary)
+   print(self.type..':',bold..self.name,self.summary..default)
    if self.description then print(self.description) end
    dump_tags (self.tags)
    for item in self.items:iter() do
@@ -1272,13 +1298,15 @@ function Item:dump(verbose)
    end
    if verbose then
       print()
+      io.write(bold)
       print(self.type,name)
+      io.write(default)
       print(self.summary)
       if self.description and self.description:match '%S' then
          print 'description:'
          print(self.description)
       end
-      if #self.params > 0 then
+      if self.params and #self.params > 0 then
          print 'parameters:'
          for _,p in ipairs(self.params) do
             print('',p,self.params.map[p])
@@ -1292,7 +1320,7 @@ function Item:dump(verbose)
       end
       dump_tags(self.tags)
    else
-      print('* '..name..' - '..self.summary)
+      print('* '..bold..name..default..' - '..self.summary)
    end
 end
 
